#ifndef FEMELEMENTSET_H
#define FEMELEMENTSET_H

#include "fem.h"
#include <Eigen/Core>

namespace fem
{
    
/// a set of elements of a given type

class FEM_API ElementSet
{
public:
    int type_id;                      ///< type (integer) e.g. 5 => "Hexahedron 8"
    std::vector<std::size_t> etags;   ///< element tags
    std::vector<std::size_t> ntags_v; ///< node tags [nodes of elem1, then elem2, elem3, etc.]

    Eigen::Map<Eigen::Matrix<size_t, Eigen::Dynamic, Eigen::Dynamic, Eigen::RowMajor>>
        ntags;

    // element type data (could be shared)
    std::string type_name;
    int type_dim;
    int type_order;
    int type_nnods;
    std::vector<double> type_localNodeCoord;
    int type_numPrimaryNodes;

public:
    ElementSet(int _typ, std::vector<std::size_t> const &_etags,
               std::vector<std::size_t> const &_ntags);
    ~ElementSet();

    friend FEM_API std::ostream &operator<<(std::ostream &out, ElementSet const &obj);
};

} // namespace fem

#endif // FEMELEMENTSET_H
