#include "femEntity.h"
#include "femElementSet.h"
#include "femGmsh.h"

namespace fem
{

Entity::Entity(int _dim, int _tag) : dim(_dim), tag(_tag)
{
    gmsh::model::getEntityName(dim, tag, name);
    gmsh::model::getType(dim, tag, typ);

    // mesh
    std::vector<int> elementTypes;
    std::vector<std::vector<std::size_t>> elementTags;
    std::vector<std::vector<std::size_t>> nodeTags;
    gmsh::model::mesh::getElements(elementTypes, elementTags, nodeTags, dim, tag);
    for (size_t i = 0; i < elementTypes.size(); ++i)
        esets.push_back(new ElementSet(elementTypes[i], elementTags[i], nodeTags[i]));
}

Entity::~Entity()
{
    // std::cout << "~Entity()\n";
    for (auto &e : esets)
        delete e;
}

void Entity::link_to_groups(std::map<std::pair<int, int>, Group *> const &groups_by_dimtag)
{
    std::vector<int> tags;
    gmsh::model::getPhysicalGroupsForEntity(dim, tag, tags);
    for (auto tag : tags)
    {
        try
        {
            groups.push_back(groups_by_dimtag.at(std::make_pair(dim, tag)));
        }
        catch (const std::exception &e)
        {
            throw std::runtime_error("Entity::link_to_groups: entity (" + std::to_string(dim) + "," + std::to_string(tag) + ") not found!");
        }
    }
}

FEM_API std::ostream &
operator<<(std::ostream &out, Entity const &obj)
{
    out << obj.typ << " (" << obj.dim << "d, " << obj.tag << ") named \""
        << obj.name << "\" with " << obj.esets.size() << " mesh(es), belonging to "
        << obj.groups.size() << " group(s)";
    return out;
}

} // namespace fem
